package gregapi.block.prefixblock;

import static gregapi.data.CS.F;
import static gregapi.data.CS.NW_API;
import static gregapi.data.CS.T;
import static gregapi.data.CS.W;
import gregapi.network.packets.data.PacketSyncDataName;
import gregapi.network.packets.data.PacketSyncDataShort;
import gregapi.render.IRenderedBlockObject;
import gregapi.render.ITexture;
import gregapi.tileentity.ITileEntitySynchronising;
import gregapi.tileentity.base.TileEntityBase01Root;
import gregapi.util.UT;
import net.minecraft.block.Block;
import net.minecraft.client.renderer.RenderBlocks;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.Packet;

/**
 * @author Gregorius Techneticies
 */
public class PrefixBlockTileEntity extends TileEntityBase01Root implements IRenderedBlockObject, ITileEntitySynchronising {
	public short mMetaData = W;
	public boolean mBlocked = true, mHasUpdateToBeSent = true;
	public NBTTagCompound mItemNBT = null;
	
	public PrefixBlockTileEntity() {super(false);}
	
    public void sendInitialUpdate() {
    	if (mHasUpdateToBeSent && !(mBlocked = UT.Worlds.isBlockOccluded(worldObj, xCoord, yCoord, zCoord, F, T))) {
    		mHasUpdateToBeSent = false;
    		NW_API.sendToAllPlayersInRange(new PacketSyncDataShort(xCoord, yCoord, zCoord, mMetaData), worldObj, xCoord, zCoord);
    		if (mItemNBT != null && mItemNBT.hasKey("display")) NW_API.sendToAllPlayersInRange(new PacketSyncDataName(xCoord, yCoord, zCoord, mItemNBT.getCompoundTag("display").getString("Name")), worldObj, xCoord, zCoord);
    	}
    }
    
    @Override
    public void onAdjacentBlockChange(int aX, int aY, int aZ) {
    	if (!worldObj.isRemote && mBlocked) {
    		mBlocked = false;
    		NW_API.sendToAllPlayersInRange(new PacketSyncDataShort(xCoord, yCoord, zCoord, mMetaData), worldObj, xCoord, zCoord);
    		if (mItemNBT != null && mItemNBT.hasKey("display")) NW_API.sendToAllPlayersInRange(new PacketSyncDataName(xCoord, yCoord, zCoord, mItemNBT.getCompoundTag("display").getString("Name")), worldObj, xCoord, zCoord);
    	}
    }
    
	@Override
	public void sendUpdateToPlayer(EntityPlayerMP aPlayer) {
		if (!(mBlocked = UT.Worlds.isBlockOccluded(worldObj, xCoord, yCoord, zCoord, T, T))) {
			NW_API.sendToPlayer(new PacketSyncDataShort(xCoord, yCoord, zCoord, mMetaData), aPlayer);
			if (mItemNBT != null && mItemNBT.hasKey("display")) NW_API.sendToPlayer(new PacketSyncDataName(xCoord, yCoord, zCoord, mItemNBT.getCompoundTag("display").getString("Name")), aPlayer);
		}
	}
	
	@Override
    public Packet getDescriptionPacket() {
		return null;
    }
	
	private ITexture mTexture;
	
	@Override
	public ITexture getTexture(Block aBlock, int aRenderPass, byte aSide) {
		if (mTexture == null) mTexture = (aBlock instanceof PrefixBlock ? ((PrefixBlock)aBlock).getTexture(mMetaData, worldObj != null) : null);
		return mTexture;
	}
	
	@Override public boolean renderItem(Block aBlock, RenderBlocks aRenderer) {return false;}
	@Override public boolean renderBlock(Block aBlock, RenderBlocks aRenderer) {return false;}
	@Override public boolean setBlockBounds(Block aBlock, int aRenderPass) {return false;}
	@Override public int getRenderPasses(Block aBlock) {return 1;}
	@Override public void readFromNBT(NBTTagCompound aNBT) {super.readFromNBT(aNBT); mMetaData = aNBT.getShort("m"); if (aNBT.hasKey("gt.nbt.drop")) mItemNBT = aNBT.getCompoundTag("gt.nbt.drop");}
    @Override public void writeToNBT(NBTTagCompound aNBT) {super.writeToNBT(aNBT); aNBT.setShort("m", mMetaData); if (mItemNBT != null && !mItemNBT.hasNoTags()) aNBT.setTag("gt.nbt.drop", mItemNBT);}
	@Override public Object getGUIClient(int aGUIID, EntityPlayer aPlayer) {return null;}
	@Override public Object getGUIServer(int aGUIID, EntityPlayer aPlayer) {return null;}
}