package gregapi.tileentity.connectors;

import static gregapi.data.CS.*;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_GetComparatorInputOverride;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_GetLightValue;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_IsProvidingStrongPower;
import gregapi.block.multitileentity.IMultiTileEntity.IMTE_IsProvidingWeakPower;
import gregapi.code.TagData;
import gregapi.data.LH;
import gregapi.data.LH.Chat;
import gregapi.data.OP;
import gregapi.data.TD;
import gregapi.old.Textures;
import gregapi.render.BlockTextureDefault;
import gregapi.render.ITexture;
import gregapi.tileentity.data.ITileEntityProgress;
import gregapi.tileentity.delegate.DelegatorTileEntity;
import gregapi.util.UT;

import java.util.Collection;
import java.util.List;

import net.minecraft.block.Block;
import net.minecraft.block.BlockRedstoneWire;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;

/**
 * @author Gregorius Techneticies
 */
public class MultiTileEntityWireRedstone extends TileEntityBase10ConnectorRendered implements ITileEntityRedstoneWire, ITileEntityProgress, IMTE_IsProvidingWeakPower, IMTE_IsProvidingStrongPower, IMTE_GetComparatorInputOverride, IMTE_GetLightValue {
	public static final int REDSTONE_ID = -1;
	
	public long mRedstone = 0, mLoss = 1;
	public byte mRenderType = 0, mReceived = SIDE_UNDEFINED, mState = 0;
	public boolean mConnectedToNonWire = F;
	
	@Override
	public void readFromNBT2(NBTTagCompound aNBT) {
    	super.readFromNBT2(aNBT);
    	if (aNBT.hasKey("gt.mreceived")) mReceived = aNBT.getByte("gt.mreceived");
    	if (aNBT.hasKey("gt.mredstone")) mRedstone = aNBT.getByte("gt.mredstone");
    	if (aNBT.hasKey(NBT_PIPELOSS)) mLoss = Math.max(1, aNBT.getLong(NBT_PIPELOSS));
    	if (aNBT.hasKey(NBT_PIPERENDER)) mRenderType = aNBT.getByte(NBT_PIPERENDER);
    	if (aNBT.hasKey(NBT_STATE)) mState = aNBT.getByte(NBT_STATE);
	}
	
	@Override
	public void writeToNBT2(NBTTagCompound aNBT) {
    	super.writeToNBT2(aNBT);
		aNBT.setByte(NBT_STATE, mState);
		aNBT.setByte("gt.mreceived", mReceived);
		UT.NBT.setNumber(aNBT, "gt.mredstone", mRedstone);
	}
	
	@Override
	public void addToolTips(List aList, ItemStack aStack, boolean aF3_H) {
		aList.add(Chat.CYAN + LH.get(LH.PIPE_STATS_RANGE) + (MAX_RANGE / mLoss));
		aList.add(Chat.CYAN + LH.get(LH.PIPE_STATS_BANDWIDTH) + 1);
		super.addToolTips(aList, aStack, aF3_H);
	}
	
    @Override
    public void onTickFirst(boolean aIsServerSide) {
    	super.onTickFirst(aIsServerSide);
    	updateConnectionStatus();
    }
    
    @Override
	public void onConnectionChange() {
    	super.onConnectionChange();
    	updateConnectionStatus();
		if (updateRedstone(REDSTONE_ID)) ITileEntityRedstoneWire.Util.doRedstoneUpdate(this, REDSTONE_ID);
	}
	
    @Override
    public void onTick2(long aTimer, boolean aIsServerSide) {
    	super.onTick2(aTimer, aIsServerSide);
    	
	    if (aIsServerSide) {
	    	if (mBlockUpdated) updateConnectionStatus();
	    	if (updateRedstone(REDSTONE_ID)) ITileEntityRedstoneWire.Util.doRedstoneUpdate(this, REDSTONE_ID);
	    }
    }
    
    @Override
    public boolean onTickCheck(long aTimer) {
    	byte tOldState = mState;
    	mState = UT.Code.bind4(mRedstone / MAX_RANGE);
    	if (tOldState != mState) {
    		updateLightValue();
    		return T;
    	}
    	return super.onTickCheck(aTimer);
    }
    
	@Override
	public void setVisualData(byte aData) {
		if (aData != mState) {
			mState = aData;
    		if (mIsGlowing) updateLightValue();
    	}
	}
	
	@Override
	public byte getVisualData() {
		return mState;
	}
	
	@Override
	public int getLightOpacity() {
		return mIsGlowing ? LIGHT_OPACITY_NONE : super.getLightOpacity();
	}
	
    public long getRedstoneAtSide(byte aSide) {
    	if (SIDES_INVALID[aSide]) return 0;
    	DelegatorTileEntity<TileEntity> tDelegator = getAdjacentTileEntity(aSide);
    	return tDelegator.mTileEntity instanceof ITileEntityRedstoneWire ? canAcceptRedstoneFromWire(aSide, REDSTONE_ID) && ((ITileEntityRedstoneWire)tDelegator.mTileEntity).canEmitRedstoneToWire(tDelegator.mSideOfTileEntity, REDSTONE_ID) ? ((ITileEntityRedstoneWire)tDelegator.mTileEntity).getRedstoneMinusLoss(tDelegator.mSideOfTileEntity, REDSTONE_ID) : 0 : canAcceptRedstoneFromVanilla(aSide) ? MAX_RANGE * (getRedstoneIncoming(aSide)+1) - mLoss : 0;
    }
    
    @Override
	public boolean updateRedstone(int aRedstoneID) {
    	if (aRedstoneID != REDSTONE_ID) return F;
    	long tRedstoneOld = mRedstone, tRedstone;
    	byte tReceivedOld = mReceived;
    	if ((mRedstone = getRedstoneAtSide(mReceived)) <= 0) mReceived = SIDE_UNDEFINED;
    	for (byte tSide : ALL_SIDES_VALID_BUT[tReceivedOld]) if ((tRedstone = getRedstoneAtSide(tSide)) > mRedstone) {mRedstone = tRedstone; mReceived = tSide;}
    	if (mRedstone != tRedstoneOld) {if (mConnectedToNonWire) causeBlockUpdate(); return T;}
    	return F;
    }
    
    public void updateConnectionStatus() {
    	mConnectedToNonWire = F;
    	for (byte tSide : ALL_SIDES_VALID) if (canAcceptRedstoneFromVanilla(tSide) && !(getAdjacentTileEntity(tSide).mTileEntity instanceof ITileEntityRedstoneWire)) mConnectedToNonWire = T;
    }
    
	@Override
	public int isProvidingWeakPower(byte aSide) {
		if (!canEmitRedstoneToVanilla(aSide = OPPOSITES[aSide])) return 0;
		Block tBlock = getBlockAtSide(aSide);
		return UT.Code.bind4((mRedstone / MAX_RANGE) - (tBlock instanceof BlockRedstoneWire || tBlock.isNormalCube(worldObj, xCoord+OFFSETS_X[aSide], yCoord+OFFSETS_Y[aSide], zCoord+OFFSETS_Z[aSide]) ? 1 : 0));
	}
	
	@Override
	public int isProvidingStrongPower(byte aSide) {
		if (!canEmitRedstoneToVanilla(aSide = OPPOSITES[aSide])) return 0;
		Block tBlock = getBlockAtSide(aSide);
		return UT.Code.bind4((mRedstone / MAX_RANGE) - (tBlock instanceof BlockRedstoneWire || tBlock.isNormalCube(worldObj, xCoord+OFFSETS_X[aSide], yCoord+OFFSETS_Y[aSide], zCoord+OFFSETS_Z[aSide]) ? 1 : 0));
	}
	
	@Override
	public int getComparatorInputOverride(byte aSide) {
		return UT.Code.bind4(mRedstone / MAX_RANGE);
	}
	
	public boolean canEmitRedstoneToVanilla					(byte aSide) {return aSide != mReceived && connected(aSide) && !(getAdjacentTileEntity(aSide).mTileEntity instanceof ITileEntityRedstoneWire);}
	public boolean canAcceptRedstoneFromVanilla				(byte aSide) {return connected(aSide);}
	
    @Override public boolean canEmitRedstoneToWire			(byte aSide, int aRedstoneID) {return aRedstoneID == REDSTONE_ID && connected(aSide);}
	@Override public boolean canAcceptRedstoneFromWire		(byte aSide, int aRedstoneID) {return aRedstoneID == REDSTONE_ID && connected(aSide);}
	
	@Override public long getRedstoneLoss					(int aRedstoneID) {return aRedstoneID == REDSTONE_ID ? mLoss : MAX_RANGE;}
	@Override public long getRedstoneValue					(byte aSide, int aRedstoneID) {return aRedstoneID == REDSTONE_ID ? mRedstone         : 0;}
	@Override public long getRedstoneMinusLoss				(byte aSide, int aRedstoneID) {return aRedstoneID == REDSTONE_ID ? mRedstone - mLoss : 0;}
    
	@Override public boolean canConnect						(byte aSide, DelegatorTileEntity<TileEntity> aDelegator) {return T;}
	
	@Override public long getProgressValue					(byte aSide) {return (1000*mRedstone)/MAX_RANGE;}
	@Override public long getProgressMax					(byte aSide) {return 16000;}
	
	@Override public ITexture getTextureSide				(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {return new BlockTextureDefault(mMaterial, getIconIndexSide		(aSide, aConnections, aDiameter, aRenderPass), mState > 0, mRGBa);}
	@Override public ITexture getTextureConnected			(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {return new BlockTextureDefault(mMaterial, getIconIndexConnected	(aSide, aConnections, aDiameter, aRenderPass), mState > 0, mRGBa);}
	@Override public ITexture getTextureCFoam				(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {return new BlockTextureDefault(Textures.BlockIcons.CFOAM_FRESH		, mRGBa, mState > 0);}
	@Override public ITexture getTextureCFoamDry			(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {return new BlockTextureDefault(Textures.BlockIcons.CFOAM_HARDENED	, mRGBa, mState > 0);}
	
	@Override public int getIconIndexSide					(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {return OP.wire.mIconIndexBlock;}
	@Override public int getIconIndexConnected				(byte aSide, byte aConnections, float aDiameter, int aRenderPass) {return OP.wire.mIconIndexBlock;}
	
	@Override public Collection<TagData> getConnectorTypes	(byte aSide) {return TD.Connectors.ALL_WIRE_REDSTONE;}
	
	@Override public String getFacingTool					() {return TOOL_cutter;}
	@Override public int getLightValue						() {return mState;}
	
	@Override public String getTileEntityName				() {return "gt.multitileentity.connector.wire.redstone";}
}