package gregapi.tileentity.delegate;

import static gregapi.data.CS.FORGE_DIR;
import gregapi.util.UT;
import net.minecraft.block.Block;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraftforge.common.util.ForgeDirection;

/**
 * @author Gregorius Techneticies
 */
public final class DelegatorTileEntity<T> {
    /** the TileEntity. This should be an instance of TileEntity. */
	public final T mTileEntity;
	/** the Coordinates the TileEntity has, or should have in case mTileEntity is null. */
	public final int mX, mY, mZ;
	/** the World Object the TileEntity has, or should have in case mTileEntity is null. */
	public final World mWorld;
    /** the Side of the Delegate responsible for handling. So a TE-Tesseract alike can go a curve. */
	public final byte mSideOfTileEntity;

	public DelegatorTileEntity(DelegatorTileEntity<T> aDelegator) {
		mTileEntity = aDelegator.mTileEntity;
		mWorld = aDelegator.mWorld;
		mX = aDelegator.mX;
		mY = aDelegator.mY;
		mZ = aDelegator.mZ;
		mSideOfTileEntity = aDelegator.mSideOfTileEntity;
	}
	
	public DelegatorTileEntity(T aTileEntity, byte aSideOfTileEntity) {
		mTileEntity = aTileEntity;
		mSideOfTileEntity = aSideOfTileEntity;
		if (aTileEntity instanceof TileEntity) {
			mWorld = ((TileEntity)aTileEntity).getWorldObj();
			mX = ((TileEntity)aTileEntity).xCoord;
			mY = ((TileEntity)aTileEntity).yCoord;
			mZ = ((TileEntity)aTileEntity).zCoord;
		} else {
			throw new IllegalArgumentException("Parameter must be an instance of TileEntity and != null");
		}
	}
	
	public DelegatorTileEntity(T aTileEntity, DelegatorTileEntity aDelegator) {
		mTileEntity = aTileEntity;
		mWorld = aDelegator.mWorld;
		mX = aDelegator.mX;
		mY = aDelegator.mY;
		mZ = aDelegator.mZ;
		mSideOfTileEntity = aDelegator.mSideOfTileEntity;
	}
	
	public DelegatorTileEntity(T aTileEntity, World aWorld, int aX, int aY, int aZ, byte aSideOfTileEntity) {
		mTileEntity = aTileEntity;
		mWorld = aWorld;
		mX = aX;
		mY = aY;
		mZ = aZ;
		mSideOfTileEntity = aSideOfTileEntity;
	}
	
	public ForgeDirection getForgeSideOfTileEntity() {return FORGE_DIR[mSideOfTileEntity];}
	public Block getBlock() {return mWorld.getBlock(mX, mY, mZ);}
	public byte getMetaData() {return UT.Code.bind4(mWorld.getBlockMetadata(mX, mY, mZ));}
	public boolean setBlock(Block aBlock) {return mWorld.setBlock(mX, mY, mZ, aBlock, 0, 3);}
	public boolean setBlock(Block aBlock, byte aMetaData) {return mWorld.setBlock(mX, mY, mZ, aBlock, aMetaData, 3);}
	public boolean setBlock(Block aBlock, byte aMetaData, int aFlags) {return mWorld.setBlock(mX, mY, mZ, aBlock, aMetaData, aFlags);}
	public boolean setMetaData(byte aMetaData) {return mWorld.setBlockMetadataWithNotify(mX, mY, mZ, aMetaData, 3);}
	public boolean setMetaData(byte aMetaData, int aFlags) {return mWorld.setBlockMetadataWithNotify(mX, mY, mZ, aMetaData, aFlags);}
	
	public boolean equalCoords(DelegatorTileEntity aOther) {return aOther.mX == mX && aOther.mY == mY && aOther.mZ == mZ;}
	public boolean equalSideAndCoords(DelegatorTileEntity aOther) {return aOther.mSideOfTileEntity == mSideOfTileEntity && equalCoords(aOther);}
	public boolean equalSideWorldAndCoords(DelegatorTileEntity aOther) {return aOther.mWorld == mWorld && equalSideAndCoords(aOther);}
	public boolean equalSideTileEntityAndCoords(DelegatorTileEntity aOther) {return aOther.mTileEntity == mTileEntity && equalSideAndCoords(aOther);}
}